/***************************************************************************
                          sessionmanager.cpp  -  description
                             -------------------
    begin                : lun apr 28 15:20:00 CET 2003
    copyright            : (C) 2003 by Romain Vinot
    email                : vinot@aist.enst.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef _WIN32
#pragma warning (disable : 4786)
#endif

#include "sessionmanager.h"

#include "gamemanager.h"
#include "board.h"
#include "guicommandline.h"
#include "datastoring.h"
#include "level.h"
#include "network.h"

SessionManager::SessionManager(GUICommandLine *g)
  : guicmd(g)
{}

bool SessionManager::NewGame(void)
{
  if (man->isInGame()) {
    if (man->getGameType()==PBEM) {
      guicmd->printPopupMessage("You're playing a PBEM game.\nYou must finish your turn or your game will not be playable anymore.");
      return false;
    }
    if (!guicmd->GetYesOrNo("Are you sure you want\n"
		    "to lose your current game to begin a new one ?"))
      return false;
    board->removeAll();
    man->removeAll();
  }
  bool heuris;
  int cheatProtection;
  gameType gt = guicmd->GetGameType(&heuris, &cheatProtection);
  if (gt==PBEM && !heuris) {
    guicmd->Write("For PBEM, you must use the heuristic. It has been enabled by default.");
    heuris=true;
  }
  man->setGameParameters(heuris,cheatProtection);
  playerTurn=MARINE_TURN;
  if (gt==NETWORK) {
    // Waiting for the connection.
    server = new NetworkServer(guicmd);
    guicmd->Write("Waiting for the connection of your opponent...");
    return true;
  }
  man->Init(gt);
  return true;
}

bool SessionManager::Load(const char *filename)
{
  bool res=store->Load(filename);

  if (res && man->getGameType()==PBEM) {

    if (man->pbemidGenestealer==0) {
      // This is the first sending for Genestealer player.
      playerTurn = GENESTEALER_TURN;
      man->pbemidGenestealer = store->getNewPbemId();
      
      if (man->nbTurns==0) {
	// This is the initialisation phase.
	guicmd->Write("Initialisation of genestealer units");
	man->level->InitGenestealer();
	man->turn=GENESTEALER_TURN;
	store->addPbem(man->pbemidGenestealer,man->nbTurns,man->turn);
	man->nbTurns=0;
	store->SaveGameState();
	if (man->level->getFirstPlayerIsMarine()) {
	  guicmd->printPopupMessage("Your opponent will now begin the game.\nPlease save and send the file.");
	  man->SavePBEM();
	  board->removeAll();
	  man->removeAll();
	}
	else {
	  guicmd->printPopupMessage("First player is genestealer.\n"
				    "You can begin your turn.");
	  man->nbTurns=1;
	  man->InitPBEM();
	  man->beginTurn();
	}
	return true;
      }

      if (man->nbTurns==1 && man->level->isInitGenestealerNull()) {
	//There was no initialisation, so this is already first turn.
	guicmd->Write("Game successfully loaded.\n"
		      "Going to replay the last turn.");
	store->addPbem(man->pbemidGenestealer, 0, GENESTEALER_TURN);
	man->Replay();
	guicmd->Write("\nReplay has been done. You can now play your turn.");
	if (man->turn==MARINE_TURN)
	  // First turn is for genestealer : replay has not done a endTurn.
	  man->endTurn();
	man->InitPBEM();
	man->beginTurn();
	store->updatePbem(man->pbemidGenestealer, man->nbTurns, man->turn);
	return true;
      }
    }

    int pbemid;
    if (man->turn==MARINE_TURN) {
      pbemid = man->pbemidGenestealer;
      playerTurn = GENESTEALER_TURN;
    }
    else {
      pbemid = man->pbemidMarine;
      playerTurn = MARINE_TURN;
    }

    pair<int,Turn> *storedPbem = store->getPbemFromList(pbemid);
    
    //   if (0) // Anticheat disabled...
    if (man->cheatProtection < 3 &&
	(storedPbem == 0 ||
	 storedPbem->second==man->turn ||
	 (storedPbem->second==MARINE_TURN&&storedPbem->first!=man->nbTurns) ||
	 ((storedPbem->second==GENESTEALER_TURN||storedPbem->second==CAT_TURN)
	  && storedPbem->first!=man->nbTurns-1))) {
      if (man->cheatProtection==1) {
	board->removeAll();
	man->removeAll();
	man->inGame = false;
	guicmd->printPopupMessage("You are not allowed to play "
				  "this PBEM game.");
	return false;
      }
      man->hasCheated=true;
    }
    guicmd->Write("Game successfully loaded.\nGoing to replay the last turn.");
    man->Replay();
    guicmd->Write("\nReplay has been done. You can now play your turn.");
    if (man->nbTurns==0) { // Otherwise done by the Replay command.
      man->nbTurns=1;
      man->turn=MARINE_TURN;
    }
    man->InitPBEM();
    man->beginTurn();
    store->updatePbem(pbemid, man->nbTurns, man->turn);
    return true;
  }

  if (res) {
    guicmd->Write("Your game has been successfully loaded.");
    man->Replay();
    return true;
  }
  else {
    guicmd->Write("A problem has occured during the load process.");
    return false;
  }
}

bool SessionManager::Save(const char *filename)
{
  if (man->getGameType()==PBEM) {
    guicmd->printPopupMessage("You can't save during PBEM game.");
    guicmd->Write("Your PBEM game has not been saved.");
    return false;
  }
  else {
    bool res=store->Save(filename);
    if (res) {
      guicmd->Write("Your game has been saved.");
      return true;
    }
    else {
      guicmd->Write("A problem has occured during the save process.\nYour game is not saved.");
      return false;
    }
  }
}


bool SessionManager::EndOfTurn(void)
{
  if (man->getGameType()==HOTSEAT)
    guicmd->commandSave("save " + homeDir + DIR_DELIMITER + "savegame" + DIR_DELIMITER + "autosave.sav");

  pair<bool,Turn> res = man->endTurn();
  Turn newt = res.second;
  if (res.first) { // This the end of the game.
    if (man->getGameType() == PBEM && !man->isInReplayMode()) {
      guicmd->printPopupMessage("You must now send and save the file "
				"for your opponent.");
      man->SavePBEM();
    }
    board->removeAll();
    man->removeAll();
    guicmd->ClearInfoPanel();
    return true;
  }
  if (man->getGameType()==PBEM) {
    Turn curr = playerTurn;
    if (curr==CAT_TURN) curr=GENESTEALER_TURN;
    if (!man->isInReplayMode() && curr!=newt) {
      // This is the end of the player turn. Store file and quit.
      guicmd->printPopupMessage("You have played your turn for this PBEM game. Please save and send the file.");
      man->SavePBEM();
      board->removeAll();
      man->removeAll();
      guicmd->ClearInfoPanel();
      return true;
    }
  }
  guicmd->ClearInfoPanel();
  switch (man->getTurn()) {
  case MARINE_TURN:
    guicmd->Write("This is the turn of the Marine player.");
    break;
  case GENESTEALER_TURN:
    guicmd->Write("This is the turn of the Genestealer player.");
    break;
  case CAT_TURN:
    guicmd->Write("This is the turn of the CAT module.");
    if (newt==MARINE_TURN)
      guicmd->Write("Cat module is free. Marine player can control it for this turn.");
    else
      guicmd->Write("Cat module is free. Genestealer player can control it for this turn.");
    break;
  }
  if(! (man->getGameType()==PBEM && man->isInReplayMode() &&
	man->it_replay==man->replays.end()) )
    // You must not run beginTurn if this is the end of the replay.
    man->beginTurn();
  return true;
}

bool SessionManager::Quit(void)

{

//  if (man->getGameType()==PBEM) {

//	 guicmd->printPopupMessage("By exiting now, you will lose the ability to continue your PBEM game.");	

//  }

  return true;

}
